module Strategize
  # Rules
  autoload :Rule,                    'strategize/rules/rule'
  autoload :RuleGroup,               'strategize/rules/rule_group'
  autoload :RuleEvaluation,          'strategize/rules/rule_evaluation'
  autoload :RuleEvaluationDigest,    'strategize/rules/rule_evaluation_digest'

  # Policies
  autoload :Policy,                  'strategize/policies/policy'
  autoload :PolicyGroup,             'strategize/policies/policy_group'
  autoload :PolicyEvaluation,        'strategize/policies/policy_evaluation'
  autoload :PolicyEvaluationDigest,  'strategize/policies/policy_evaluation_digest'

  # Evaluators
  autoload :RuleEvaluator,           'strategize/evaluators/rule_evaluator'
  autoload :PolicyEvaluator,         'strategize/evaluators/policy_evaluator'

  # Operations
  autoload :Operation,               'strategize/operations/operation'
  autoload :OperationGroup,          'strategize/operations/operation_group'

  # Exceptions
  autoload :NotPolicyError,          'strategize/exceptions'
  autoload :InvalidPredicateError,   'strategize/exceptions'
  autoload :InvalidFunctionError,    'strategize/exceptions'
  autoload :InvalidRuleEvaluator,    'strategize/exceptions'
  autoload :InvalidPolicyEvaluator,  'strategize/exceptions'

  # Configuration
  autoload :Configuration,           'strategize/configuration'

  class << self
    # Get the configuration object for Strategize
    #
    # @return [Configuration]
    def configuration
      @configuration ||= Configuration.new
    end

    # Overwrite the current [Configuration] object
    # with a new [Configuration] instance, which
    # returns everything to default
    def reset_configuration
      @configuration = Configuration.new
    end

    # Pass a block of code that is executed against the
    # [Configuration] instance
    def configure
      configuration = self.configuration
      yield(configuration)
    end
  end
end
