module Strategize
  class OperationGroup
    # @attr_reader operations [Array<Operation>]
    attr_reader :operations

    # Create a new instance of OperationGroup
    #
    # @param operations [Array<Operation>] the operations to be executed
    def initialize(operations=[])
      @operations = operations
    end

    # Add Operation to group
    #
    # @param name [Symbol] a descriptive name for the operation
    # @param operation [Proc] a proc or lambda describing the code to be executed
    def add(name, function)
      operation = Operation.new(name, function)
      @operations.push(operation)
    end

    # Execute the code definined in each operation against a subject.
    #
    # @param subject [Object] a single object which acts as the subject for each operation
    # @param subject [Hash] a hash of operation names and subjects
    def run(subjects)
      the_subjects = subjects
      the_subjects = build_subjects_hash(subjects) unless subjects.is_a?(Hash)


      @operations.each do |operation|
        subject = the_subjects[operation.name]
        operation.process(subject) unless subject.nil?
      end
    end

    private

    def build_subjects_hash(subjects)
      { default: subjects }
    end
  end
end
