module Strategize
  # The policy module allows you to define rules and operations on
  # any class that it is included on
  module Policy
    def self.included(base)
      base.extend(ClassMethods)
      base.send(:attr_reader, :object)
    end

    # These methods will be placed on the class that includes
    # the Policy module.
    module ClassMethods
      # Get the [RuleGroup] container that has all the
      # rules defined on the policy
      #
      # @return [RuleGroup]
      def rule_group
        @rule_group ||= RuleGroup.new
      end

      # Get the [OperationGroup] container that has all the
      # operations defined on the policy
      #
      # @return [OperationGroup]
      def operations
        @operations ||= OperationGroup.new
      end

      # Define and add a [Rule] to the [RuleGroup] container
      #
      # @param name [Symbol] descriptive name for the rule
      # @param predicate [Proc] code to execute
      # @return [void]
      def rule(name, predicate)
        rule_group.add(name, predicate)
      end

      # Define and add an [Operation] to the [OperationGroup] container
      #
      # @param name [Symbol] the name of the context for the operation
      # @param block [Proc] the code block to execute
      # @return [void]
      def operation(name = :default, &block)
        operations.add(name, block)
      end
    end
  end
end
