module Strategize
  # Provides information on a single policy evaluation.
  # This includes the policy that was evaluated and
  # the evaluation digest for the rules defined on
  # the policy.
  class PolicyEvaluation
    attr_reader :policy, :rule_digest

    # Create a new PolicyEvaluation
    #
    # @param policy [Policy]
    # @param rule_result_collection [RuleEvaluationDigest]
    # @return [PolicyResult]
    def initialize(policy, rule_digest)
      @policy = policy
      @rule_digest = rule_digest
    end

    # Determine if all the rules defined on the policy
    # evaluated to true.  
    # @return [Boolean]
    def result
      @rule_digest.result
    end
  end
end
