module Strategize
  # Represents a container for evaluations performed against each
  # policy provided in the initializer
  class PolicyEvaluationDigest
    # @return [Array<PolicyEvaluation>] the evaluation for each policy
    attr_reader :evaluations

    # @return [Object] the subject the policy was executed against
    attr_reader :subject

    # Create a new PolicyEvaluationDigest
    #
    # @param evaluations [Array<PolicyEvaluation>]
    # @param subject [Object] the subject the policies were executed against
    def initialize(evaluations, subject)
      @evaluations = evaluations
      @subject = subject
    end

    # Return all the policies where all the rules defined on the
    # policy evaluated to true
    #
    # @return [Array<Policy>]
    def passed
      @evaluations
        .select { |evaluation| evaluation.result == true }
        .map(&:policy)
    end

    # Return all the policies where one or more rules defined on the
    # policy evaluated to false.
    #
    # @return [Array<Policy>]
    def failed
      @evaluations
        .select { |evaluation| evaluation.result == false }
        .map(&:policy)
    end
  end
end
